import base64
import pkgutil
from Crypto.Cipher import PKCS1_OAEP
from Crypto.PublicKey import RSA

class RsaUtil(object):

    @staticmethod
    def encrypt(key):
        public_key_file = pkgutil.get_data(__name__, "templates/public.der")
        public_key = RSA.importKey(public_key_file)
        # print(public_key.exportKey().decode('ascii'))

        cipher_rsa = PKCS1_OAEP.new(public_key)
        encrypted = cipher_rsa.encrypt(str.encode(key))

        return base64.b64encode(encrypted).decode('ascii')




from takeme.TakemeRequestor import TakemeRequestor
from takeme.RsaUtil import RsaUtil


class TakemeClient:

    def __init__(self, corporate_code, secret_key):
        self.request_takeme = TakemeRequestor(corporate_code, secret_key)

    def non_auth_call(self, body, endpoint):
        response = self.request_takeme.call_api_without_bearer(endpoint, body)

        if response.status_code != 200:
            raise ValueError(response.json())

        return response.json()

    def auth_call(self, body, endpoint, jwt):
        response = self.request_takeme.call_api_with_bearer(endpoint, body, jwt)

        if response.status_code != 200:
            raise ValueError(response.json())

        return response.json()

    def auth_call_with_pin(self, body, endpoint, jwt, pin):
        encrypted_pin = RsaUtil.encrypt(pin)
        response = self.request_takeme.call_api_with_bearer_and_pin(endpoint, body, jwt, encrypted_pin)

        if response.status_code != 200:
            raise ValueError(response.json())

        return response.json()

    def user_signup(self, full_name, email, phone_number):
        endpoint = "uaa/signup"
        body = {
            "full_name": full_name,
            "email": email,
            "phone_number": phone_number
        }

        self.non_auth_call(body, endpoint)

    def user_activation(self, email, phone_number, activation_code):
        endpoint = "uaa/activation"
        body = {
            "email": email,
            "phone_number": phone_number,
            "activation_code": activation_code,
        }

        response = self.non_auth_call(body, endpoint)

        return response.get("data")

    def user_prelogin(self, phone_number):
        endpoint = "uaa/prelogin"
        body = {
            "phone_number": phone_number,
        }

        self.non_auth_call(body, endpoint)

    def user_login(self, phone_number, login_code):
        endpoint = "uaa/login"
        body = {
            "phone_number": phone_number,
            "login_code": login_code,
        }

        response = self.non_auth_call(body, endpoint)

        return response.get("data").get("jwt")

    def user_save_pin(self, pin, jwt):
        endpoint = "user/save-pin"
        body = {
            "pin": rsa_util.RsaUtil.encrypt(pin)
        }

        self.auth_call(body, endpoint, jwt)

    def user_pre_forgot_pin(self, new_pin, jwt):
        endpoint = "user/pre-forgot-pin"
        body = {
            "pin": rsa_util.RsaUtil.encrypt(new_pin)
        }

        self.auth_call(body, endpoint, jwt)

    def user_confirm_forgot_pin(self, otp, jwt):
        endpoint = "user/forgot-pin"
        body = {
            "confirm_pin_code": otp
        }

        self.auth_call(body, endpoint, jwt)

    def user_change_pin(self, old_pin, new_pin, jwt):
        endpoint = "user/save-pin"
        body = {
            "old_pin": rsa_util.RsaUtil.encrypt(old_pin),
            "new_pin": rsa_util.RsaUtil.encrypt(new_pin)
        }

        self.auth_call(body, endpoint, jwt)

    def user_detail(self, jwt):
        endpoint = "user/check"
        body = None

        response = self.auth_call(body, endpoint, jwt)

        return response.get("data")

    def user_balance(self, jwt):
        endpoint = "user/check"
        body = None

        response = self.auth_call(body, endpoint, jwt)

        return response.get("data").get("balance")

    def user_transaction_history(self, jwt, offset=1, limit=10):
        endpoint = "transaction/?" + "page=" + str(offset) + "&limit=" + str(limit)
        body = None

        response = self.auth_call(body, endpoint, jwt)

        return response.get("data")

    def user_inquiry_bank_account(self, account_number, bank_code, jwt):
        endpoint = "transaction/account-holdername"
        body = {
            "account_number": account_number,
            "bank_code": bank_code
        }

        response = self.auth_call(body, endpoint, jwt)

        if not response.get("data").get("account_name"):
            raise ValueError("Invalid account number")
        else:
            return response.get("data")

    def set_bank_account(self, account_number, bank_code, jwt):
        response = self.user_inquiry_bank_account(account_number, bank_code, jwt)

        bank_account = {
            "name": response.get("account_name"),
            "account_number": response.get("account_number"),
            "bank_code": response.get("bank_name")
        }

        return bank_account

    def transfer_to_bank(self, bank_account, amount, pin, jwt):
        endpoint = "transaction/wallet"
        body = {
            "to_bank_account": bank_account,
            "type": "",
            "notes": "",
            "amount": amount
        }

        response = self.auth_call_with_pin(body, endpoint, jwt, pin)

        return response.get("data")



import requests
import uuid
import hmac
import json
import hashlib


class TakemeRequestor(object):

    BASE_URL = "https://api.takeme.id/api/v1/"

    def __init__(self, corporate_code, secret_key):
        self.corporate_code = corporate_code
        self.secret_key = secret_key
        self.method_post = "POST"
        self.method_get = "GET"

    def call_api_without_bearer(self, endpoint, body):

        data = json.dumps(body)
        request_id = uuid.uuid4();
        byte_key = bytes(self.secret_key, 'utf-8')
        byte_data = bytes(data, 'utf-8')
        signature = hmac.new(byte_key, byte_data, hashlib.sha512).hexdigest()
        headers = {
            "Accept-Language": "en",
            "corporate": self.corporate_code,
            "requestID": str(request_id),
            "signature": signature,
        }
        if body is not None:
            response = requests.post(self.BASE_URL + endpoint, data=data, headers=headers)
            return response
        else:
            response = requests.get(self.BASE_URL + endpoint, data=data, headers=headers)
            return response

    def call_api_with_bearer(self, endpoint, body, jwt):

        data = json.dumps(body)
        request_id = uuid.uuid4();
        byte_key = bytes(self.secret_key, 'utf-8')
        byte_data = bytes(data, 'utf-8')
        signature = hmac.new(byte_key, byte_data, hashlib.sha512).hexdigest()
        headers = {
            "Accept-Language": "en",
            "corporate": self.corporate_code,
            "requestID": str(request_id),
            "signature": signature,
            "Authorization": "Bearer "+ jwt,
        }
        if body is not None:
            response = requests.post(self.BASE_URL + endpoint, data=data, headers=headers)
            return response
        else:
            response = requests.get(self.BASE_URL + endpoint, data=data, headers=headers)
            return response

    def call_api_with_bearer_and_pin(self, endpoint, body, jwt, pin):

        data = json.dumps(body)
        request_id = uuid.uuid4();
        byte_key = bytes(self.secret_key, 'utf-8')
        byte_data = bytes(data, 'utf-8')
        signature = hmac.new(byte_key, byte_data, hashlib.sha512).hexdigest()
        headers = {
            "Accept-Language": "en",
            "corporate": self.corporate_code,
            "requestID": str(request_id),
            "signature": signature,
            "Authorization": "Bearer "+ jwt,
            "x-transaction-code": pin,
        }
        if body is not None:
            response = requests.post(self.BASE_URL + endpoint, data=data, headers=headers)
            return response
        else:
            response = requests.get(self.BASE_URL + endpoint, data=data, headers=headers)
            return response




from setuptools import setup, find_packages

VERSION = '0.0.16'
DESCRIPTION = 'Python library for client who want to connect with takeme-service'

# Setting up
setup(
    name="takeme-client-python",
    version=VERSION,
    author="Restravity (Ciputra Dhimas)",
    author_email="<ciputra.developer@gmail.com>",
    description=DESCRIPTION,
    packages=find_packages(),
    include_package_data=True,
    install_requires=['certifi', 'charset-normalizer', 'idna', 'pycryptodome', 'requestor',
                      'requests', 'unicode', 'urllib3'],
    keywords=['takeme', 'takeme client', 'python', 'takeme-client-python'],
    classifiers=[
        "Development Status :: 1 - Planning",
        "Intended Audience :: Developers",
        "Programming Language :: Python :: 3",
        "Operating System :: Unix",
        "Operating System :: MacOS :: MacOS X",
        "Operating System :: Microsoft :: Windows",
    ]
)